@@ -1,3 +1,11 @@
+//! [![github]](https://github.com/dtolnay/rustversion)&ensp;[![crates-io]](https://crates.io/crates/rustversion)&ensp;[![docs-rs]](https://docs.rs/rustversion)
+//!
+//! [github]: https://img.shields.io/badge/github-8da0cb?style=for-the-badge&labelColor=555555&logo=github
+//! [crates-io]: https://img.shields.io/badge/crates.io-fc8d62?style=for-the-badge&labelColor=555555&logo=rust
+//! [docs-rs]: https://img.shields.io/badge/docs.rs-66c2a5?style=for-the-badge&labelColor=555555&logo=docs.rs
+//!
+//! <br>
+//!
 //! This crate provides macros for conditional compilation according to rustc
 //! compiler version, analogous to [`#[cfg(...)]`][cfg] and
 //! [`#[cfg_attr(...)]`][cfg_attr].
@@ -137,118 +145,104 @@
 //!
 //! <br>
 
+#![doc(html_root_url = "https://docs.rs/rustversion/1.0.13")]
+#![allow(
+    clippy::cast_lossless,
+    clippy::cast_possible_truncation,
+    clippy::derive_partial_eq_without_eq,
+    clippy::doc_markdown,
+    clippy::enum_glob_use,
+    clippy::from_iter_instead_of_collect,
+    // https://github.com/rust-lang/rust-clippy/issues/8539
+    clippy::iter_with_drain,
+    clippy::module_name_repetitions,
+    clippy::must_use_candidate,
+    clippy::needless_doctest_main,
+    clippy::needless_pass_by_value,
+    clippy::redundant_else,
+    clippy::toplevel_ref_arg,
+    clippy::unreadable_literal
+)]
+
 extern crate proc_macro;
 
 mod attr;
 mod bound;
+mod constfn;
 mod date;
+mod error;
+mod expand;
 mod expr;
+mod iter;
+mod release;
 mod time;
+mod token;
 mod version;
 
-use crate::attr::Then;
-use crate::date::Date;
-use crate::expr::Expr;
-use crate::version::{Channel::*, Version};
+use crate::error::Error;
+use crate::version::Version;
 use proc_macro::TokenStream;
-use proc_macro2::{Ident, Span, TokenStream as TokenStream2};
-use quote::quote;
-use syn::{parse_macro_input, ItemFn, Result};
 
-const RUSTVERSION: Version = include!(concat!(env!("OUT_DIR"), "/version.rs"));
+const RUSTVERSION: Version = include!(concat!(env!("OUT_DIR"), "/version.expr"));
 
 #[proc_macro_attribute]
 pub fn stable(args: TokenStream, input: TokenStream) -> TokenStream {
-    cfg("stable", args, input)
+    expand::cfg("stable", args, input)
 }
 
 #[proc_macro_attribute]
 pub fn beta(args: TokenStream, input: TokenStream) -> TokenStream {
-    cfg("beta", args, input)
+    expand::cfg("beta", args, input)
 }
 
 #[proc_macro_attribute]
 pub fn nightly(args: TokenStream, input: TokenStream) -> TokenStream {
-    cfg("nightly", args, input)
+    expand::cfg("nightly", args, input)
 }
 
 #[proc_macro_attribute]
 pub fn since(args: TokenStream, input: TokenStream) -> TokenStream {
-    cfg("since", args, input)
+    expand::cfg("since", args, input)
 }
 
 #[proc_macro_attribute]
 pub fn before(args: TokenStream, input: TokenStream) -> TokenStream {
-    cfg("before", args, input)
+    expand::cfg("before", args, input)
 }
 
 #[proc_macro_attribute]
 pub fn not(args: TokenStream, input: TokenStream) -> TokenStream {
-    cfg("not", args, input)
+    expand::cfg("not", args, input)
 }
 
 #[proc_macro_attribute]
 pub fn any(args: TokenStream, input: TokenStream) -> TokenStream {
-    cfg("any", args, input)
+    expand::cfg("any", args, input)
 }
 
 #[proc_macro_attribute]
 pub fn all(args: TokenStream, input: TokenStream) -> TokenStream {
-    cfg("all", args, input)
-}
-
-fn cfg(top: &str, args: TokenStream, input: TokenStream) -> TokenStream {
-    match try_cfg(top, args, input) {
-        Ok(tokens) => tokens,
-        Err(err) => TokenStream::from(err.to_compile_error()),
-    }
-}
-
-fn try_cfg(top: &str, args: TokenStream, input: TokenStream) -> Result<TokenStream> {
-    let args = TokenStream2::from(args);
-    let top = Ident::new(top, Span::call_site());
-
-    let mut full_args = quote!(#top);
-    if !args.is_empty() {
-        full_args.extend(quote!((#args)));
-    }
-
-    let expr: Expr = syn::parse2(full_args)?;
-
-    if expr.eval(RUSTVERSION) {
-        Ok(input)
-    } else {
-        Ok(TokenStream::new())
-    }
+    expand::cfg("all", args, input)
 }
 
 #[proc_macro_attribute]
 pub fn attr(args: TokenStream, input: TokenStream) -> TokenStream {
-    let args = parse_macro_input!(args as attr::Args);
-
-    match try_attr(args, input) {
-        Ok(tokens) => tokens,
-        Err(err) => TokenStream::from(err.to_compile_error()),
-    }
+    attr::parse(args)
+        .and_then(|args| expand::try_attr(args, input))
+        .unwrap_or_else(Error::into_compile_error)
 }
 
-fn try_attr(args: attr::Args, input: TokenStream) -> Result<TokenStream> {
-    if !args.condition.eval(RUSTVERSION) {
-        return Ok(input);
-    }
-
-    match args.then {
-        Then::Const(const_token) => {
-            let mut input: ItemFn = syn::parse(input)?;
-            input.sig.constness = Some(const_token);
-            Ok(TokenStream::from(quote!(#input)))
-        }
-        Then::Attribute(then) => {
-            let input = TokenStream2::from(input);
-            Ok(TokenStream::from(quote! {
-                #[cfg_attr(all(), #then)]
-                #input
-            }))
-        }
-    }
+#[cfg(not(cfg_macro_not_allowed))]
+#[proc_macro]
+pub fn cfg(input: TokenStream) -> TokenStream {
+    use proc_macro::{Ident, Span, TokenTree};
+    (|| {
+        let ref mut args = iter::new(input);
+        let expr = expr::parse(args)?;
+        token::parse_end(args)?;
+        let boolean = expr.eval(RUSTVERSION);
+        let ident = Ident::new(&boolean.to_string(), Span::call_site());
+        Ok(TokenStream::from(TokenTree::Ident(ident)))
+    })()
+    .unwrap_or_else(Error::into_compile_error)
 }
